@@ -1,4 +1,4 @@
-/*  $Id: BurgerSpaceEngine.cpp,v 1.4 2002/04/03 06:11:27 sarrazip Exp $
+/*  $Id: BurgerSpaceEngine.cpp,v 1.11 2003/01/11 01:41:15 sarrazip Exp $
     BurgerSpaceEngine.cpp - Main engine
 
     burgerspace - A hamburger-smashing video game.
@@ -44,7 +44,6 @@
 
 #include <map>
 #include <vector>
-#include <string>
 #include <algorithm>
 #include <iomanip>
 
@@ -187,7 +186,7 @@ static const int
     FRAMES_PER_SECOND = 20,
     TILE_SIDE = 32,
     FLOOR_POS = TILE_SIDE - 6,  // position of the floor's surface, in a tile
-    INGREDIENT_FALL_SPEED = 4,
+    INGREDIENT_FALL_SPEED = 8,
     PLAYER_SPEED_FACTOR = 4,
     ENEMY_SPEED_FACTOR = 3,
     SCORE_TTL = 20,
@@ -305,6 +304,27 @@ getDirectionName(int d)
 }
 
 
+static
+string
+getDir(const char *defaultValue, const char *envVarName)
+/*
+    Makes sure that the returned directory name ends with a slash.
+*/
+{
+    string dir;
+    const char *s = getenv(envVarName);
+    if (s != NULL)
+	dir = s;
+    else
+	dir = defaultValue;
+    
+    if (!dir.empty() && dir[dir.length() - 1] != '/')
+	dir += '/';
+
+    return dir;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // STATIC DATA MEMBERS
@@ -846,9 +866,11 @@ BurgerSpaceEngine::Level::getXPMAtPixel(Couple pos) const
 
 
 BurgerSpaceEngine::BurgerSpaceEngine(const string &windowManagerCaption,
-							int initLevelNumber)
-							    throw(int, string)
-  : GameEngine(Couple(672, 520), windowManagerCaption, false),
+					int initLevelNumber,
+					bool useSound,
+					bool fullScreen)
+						throw(int, string)
+  : GameEngine(Couple(672, 520), windowManagerCaption, fullScreen),
   		// may throw string exception
 
     initLevelNo(1),
@@ -913,8 +935,12 @@ BurgerSpaceEngine::BurgerSpaceEngine(const string &windowManagerCaption,
     numAvailablePeppers(0),
     updateNumAvailablePeppers(true),
 
-    theCurrentLevel()
+    theCurrentLevel(),
+
+    theSoundMixer(NULL)
 {
+    this->useSound = useSound;
+
     assert(initLevelNumber >= 1);
     initLevelNo = initLevelNumber;
     cumulLevelNo = initLevelNo;
@@ -1050,6 +1076,37 @@ BurgerSpaceEngine::initializeMisc() throw(string)
     levelNoAreaSize = Couple(66, 16);
     levelNoAreaPos =
 	Couple(getScreenWidthInPixels() - levelNoAreaSize.x, numLivesAreaPos.y);
+
+
+    /*  Sound effects:
+    */
+    if (useSound)
+    {
+	try
+	{
+	    string d = getDir(PKGSOUNDDIR, "PKGSOUNDDIR");
+	    theSoundMixer = new SoundMixer(16);  // may throw string
+
+	    ingredientBouncesSound.init(d + "ingredient-bounces.wav");
+	    ingredientInPlateSound.init(d + "ingredient-in-plate.wav");
+	    ingredientFallsSound  .init(d + "ingredient-falls.wav");
+	    ingredientLoweredSound.init(d + "ingredient-lowered.wav");
+	    enemyCatchesChefSound .init(d + "enemy-catches-chef.wav");
+	    enemyParalyzedSound   .init(d + "enemy-paralyzed.wav");
+	    enemySmashedSound     .init(d + "enemy-smashed.wav");
+	    chefThrowsPepperSound .init(d + "chef-throws-pepper.wav");
+	    chefGetsTreatSound    .init(d + "chef-gets-treat.wav");
+	    chefShootsBlanksSound .init(d + "chef-shoots-blanks.wav");
+	    newGameStartsSound    .init(d + "new-game-starts.wav");
+	    levelFinishedSound    .init(d + "level-finished.wav");
+	    treatAppearsSound     .init(d + "treat-appears.wav");
+	    treatDisappearsSound  .init(d + "treat-disappears.wav");
+	}
+	catch (const SoundMixer::Error &e)
+	{
+	    throw e.what();
+	}
+    }
 }
 
 
@@ -1457,6 +1514,8 @@ BurgerSpaceEngine::animatePlayer()
 	*/
 	if (!spacePressed && spacePressedBefore)
 	{
+	    playSoundEffect(newGameStartsSound);
+
 	    initGameParameters();
 	    startNewLife();
 
@@ -1477,34 +1536,44 @@ BurgerSpaceEngine::animatePlayer()
 
     /*  Shoot if requested:
     */
-    if (numAvailablePeppers > 0 && isPepperRequested())
+    static const bool infinitePepper = (getenv("INFINITEPEPPER") != NULL);
+    if (isPepperRequested())
     {
-	const Couple size = pepperPA.getImageSize();
-	const Couple plsize = playerSprite->getSize();
-	Couple pos = playerSprite->getPos();
-	int dir;
-
-	if (lastPlayerDirection == -1)
-	    dir = UP;
-	else
-	    dir = lastPlayerDirection;
-	numAvailablePeppers--;
-
-	switch (dir)
+	if ((infinitePepper || numAvailablePeppers > 0) && isPepperRequested())
 	{
-	    case UP   :  pos += Couple(0, -size.y);  break;
-	    case DOWN :  pos += Couple(0, plsize.y); break;
-	    case LEFT :  pos += Couple(-size.x, 0);  break;
-	    case RIGHT:  pos += Couple(plsize.x, 0); break;
-	    default:     assert(false);
+	    const Couple size = pepperPA.getImageSize();
+	    const Couple plsize = playerSprite->getSize();
+	    Couple pos = playerSprite->getPos();
+	    int dir;
+
+	    if (lastPlayerDirection == -1)
+		dir = UP;
+	    else
+		dir = lastPlayerDirection;
+	    numAvailablePeppers--;
+
+	    switch (dir)
+	    {
+		case UP   :  pos += Couple(0, -size.y);  break;
+		case DOWN :  pos += Couple(0, plsize.y); break;
+		case LEFT :  pos += Couple(-size.x, 0);  break;
+		case RIGHT:  pos += Couple(plsize.x, 0); break;
+		default:     assert(false);
+	    }
+
+	    playSoundEffect(chefThrowsPepperSound);
+
+	    Sprite *s = new Sprite(pepperPA, pos, Couple(0, 0),
+				    Couple(0, 0), Couple(0, 0), size);
+	    s->setTimeToLive(12);
+	    pepperSprites.push_back(s);
+
+	    updateNumAvailablePeppers = true;
+	}
+	else
+	{
+	    playSoundEffect(chefShootsBlanksSound);
 	}
-
-	Sprite *s = new Sprite(pepperPA, pos, Couple(0, 0),
-				Couple(0, 0), Couple(0, 0), size);
-	s->setTimeToLive(12);
-	pepperSprites.push_back(s);
-
-	updateNumAvailablePeppers = true;
     }
 
 
@@ -1849,11 +1918,12 @@ BurgerSpaceEngine::animateAutomaticCharacters()
 	{
 	    delete *it;
 	    *it = NULL;
+
+	    playSoundEffect(treatDisappearsSound);
 	}
     removeNullElementsFromSpriteList(treatSprites);
 
-    bool newTreat;
-    newTreat = (--timeForTreat == 0);
+    bool newTreat = (--timeForTreat == 0);
 
     if (newTreat)
     {
@@ -1878,6 +1948,8 @@ BurgerSpaceEngine::animateAutomaticCharacters()
 	s->currentPixmapIndex = spriteNo;
 	s->setTimeToLive(5 * 20);
 	treatSprites.push_back(s);
+
+	playSoundEffect(treatAppearsSound);
     }
 }
 
@@ -2198,6 +2270,8 @@ BurgerSpaceEngine::givePlayerPepper()
 void
 BurgerSpaceEngine::makePlayerWin()
 {
+    playSoundEffect(levelFinishedSound);
+
     playerSprite->setTimeToLive(80);
     releaseAllCarriedEnemies();
 
@@ -2238,6 +2312,8 @@ BurgerSpaceEngine::detectCollisions()
     if (playerSprite->getTimeToLive() != 0)  // if player is agonizing
 	return;
 
+    bool playIngredientLoweredSound = false;
+
     for (IngredientGroupList::iterator it = ingredientGroups.begin();
 					it != ingredientGroups.end(); it++)
     {
@@ -2259,6 +2335,8 @@ BurgerSpaceEngine::detectCollisions()
 		IngredientSprite *ing = aGroup->getMember(j);
 		if (!ing->isLowered() && playerSprite->collidesWithSprite(*ing))
 		{
+		    playIngredientLoweredSound = true;
+
 		    ing->lower();
 		    if (aGroup->areAllMembersLowered())
 		    {
@@ -2270,6 +2348,8 @@ BurgerSpaceEngine::detectCollisions()
 			    case 1:   numFloorsToGo = 3; break;
 			    default:  numFloorsToGo = 5;
 			}
+
+			playSoundEffect(ingredientFallsSound);
 			aGroup->startFalling(
 					INGREDIENT_FALL_SPEED, numFloorsToGo);
 		    }
@@ -2287,6 +2367,7 @@ BurgerSpaceEngine::detectCollisions()
 	int yTarget = aGroup->getVerticalTarget();
 	if (pos.y >= yTarget)
 	{
+	    playSoundEffect(ingredientInPlateSound);
 	    pos.y = yTarget;
 	    aGroup->stop();
 	    releaseCarriedEnemies(*aGroup);
@@ -2339,7 +2420,10 @@ BurgerSpaceEngine::detectCollisions()
 	if (isIngredientSpriteOnFloor(*member0))
 	{
 	    if (aGroup->getState() == IngredientGroup::FALL1)
+	    {
 		aGroup->bounce(INGREDIENT_FALL_SPEED);
+		playSoundEffect(ingredientBouncesSound);
+	    }
 	    else
 	    {
 		assert(aGroup->getState() == IngredientGroup::FALL2);
@@ -2361,6 +2445,9 @@ BurgerSpaceEngine::detectCollisions()
 	detectCollisionBetweenIngredientGroupAndEnemyList(*aGroup,enemySprites);
     }
 
+    if (playIngredientLoweredSound)
+	playSoundEffect(ingredientLoweredSound);
+
 
     /*  Enemies:
     */
@@ -2381,6 +2468,8 @@ BurgerSpaceEngine::detectCollisions()
 
 	    *its = NULL;
 	    delete treat;
+
+	    playSoundEffect(chefGetsTreatSound);
 	    givePlayerPepper();
 	}
     }
@@ -2412,6 +2501,7 @@ BurgerSpaceEngine::detectCollisionBetweenIngredientGroupAndEnemyList(
 
 	if (ingredientGroupCollidesWithSprite(groupPos, groupSize, *anEnemy))
 	{
+	    playSoundEffect(enemySmashedSound);
 	    anEnemy->setTimeToDie(20);
 	    break;
 	}
@@ -2437,16 +2527,24 @@ BurgerSpaceEngine::detectEnemyCollisions(SpriteList &slist)
 		    s->getDisappearanceTime() != 0)
 	    continue;
 
+	// Paralyze the enemy if it touches pepper.
+	// The enemy's "time to live" attribute is used for this.
 	for (SpriteList::const_iterator itp = pepperSprites.begin();
 					itp != pepperSprites.end(); itp++)
 	    if (s->collidesWithSprite(**itp))
+	    {
+		playSoundEffect(enemyParalyzedSound);
 		s->setTimeToLive(60 + rand() % 10);
+	    }
 
 	if (s->getTimeToLive() != 0)  // if enemy just paralyzed by pepper
 	    continue;
 
 	if (numLives != 0 && s->collidesWithSprite(*playerSprite))
+	{
+	    playSoundEffect(enemyCatchesChefSound);
 	    makePlayerDie();
+	}
     }
 
     removeNullElementsFromSpriteList(slist);
@@ -2977,3 +3075,21 @@ BurgerSpaceEngine::displayMessage(const char *msg)
 	displayMessage(blank);
     writeString(msg, Couple(x, y));
 }
+
+
+void
+BurgerSpaceEngine::playSoundEffect(SoundMixer::Chunk &chunk)
+{
+    if (theSoundMixer != NULL)
+    {
+	try
+	{
+	    theSoundMixer->playChunk(chunk);
+	}
+	catch (const SoundMixer::Error &e)
+	{
+	    fprintf(stderr, "playSoundEffect: %s (chunk at %p)\n",
+			e.what().c_str(), &chunk);
+	}
+    }
+}
