@@ -1,4 +1,4 @@
-/*  $Id: sdlmain.cpp,v 1.4 2002/04/03 06:11:28 sarrazip Exp $
+/*  $Id: sdlmain.cpp,v 1.5 2003/01/11 01:41:15 sarrazip Exp $
     sdlmain.cpp - main() function for BurgerSpace
 
     burgerspace - A hamburger-smashing video game.
@@ -39,8 +39,8 @@ static struct option knownOptions[] =
     { "initial-level", required_argument, NULL, 'i' },
     { "ms-per-frame",  required_argument, NULL, 'm' },
     { "remote-chef",   no_argument,       NULL, 'r' },
-    { "server-mode",   required_argument, NULL, 's' },
-    { "client-mode",   required_argument, NULL, 'c' },
+    { "no-sound",      no_argument,       NULL, 'n' },
+    { "full-screen",   no_argument,       NULL, 'f' },
 
     { NULL, 0, NULL, 0 }  // marks the end
 };
@@ -62,7 +62,7 @@ displayHelp()
 
     printf(
 "\n"
-"Copyright (C) 2001 Pierre Sarrazin <http://sarrazip.com/>\n"
+"Copyright (C) 2001, 2002, 2003 Pierre Sarrazin <http://sarrazip.com/>\n"
 "This program is free software; you may redistribute it under the terms of\n"
 "the GNU General Public License.  This program has absolutely no warranty.\n"
     );
@@ -75,6 +75,9 @@ displayHelp()
 "--initial-level=N  Start game at level N (N >= 1) [default=1]\n"
 "--ms-per-frame=N   N milliseconds per animation frame [default=55]\n"
 "                   Min. 1, max. 1000.  50 ms means 20 frames per second\n"
+"--no-sound         Disable sound effects [default is to enable them]\n"
+"--full-screen      Attempt to use full screen mode [default is window mode]\n"
+"\n"
     );
 }
 #endif  /* __GNUG__ */
@@ -86,6 +89,8 @@ int main(int argc, char *argv[])
     */
     int initLevelNo = 1;
     int millisecondsPerFrame = 55;
+    bool useSound = true;
+    bool fullScreen = false;
 
 
     #ifdef __GNUG__
@@ -131,6 +136,14 @@ int main(int argc, char *argv[])
 		    millisecondsPerFrame = int(n);
 		}
 		break;
+	    
+	    case 'n':
+		useSound = false;
+		break;
+	    
+	    case 'f':
+		fullScreen = true;
+		break;
 
 	    case 'v':
 		displayVersionNo();
@@ -160,7 +173,8 @@ int main(int argc, char *argv[])
 
     try
     {
-	BurgerSpaceEngine theBurgerSpaceEngine("BurgerSpace", initLevelNo);
+	BurgerSpaceEngine theBurgerSpaceEngine("BurgerSpace",
+					initLevelNo, useSound, fullScreen);
 	theBurgerSpaceEngine.run(millisecondsPerFrame);
     }
     catch (const string &s)
